@@ -1,7 +1,7 @@
 
-import mu32
+import mu32.core
 
 print( 'This is Mu32 program' )
 
-mu32.__main__()
+mu32.core.__main__()
 
