@@ -49,6 +49,14 @@ from math import ceil as ceil
 from .log import mulog as log
 from .exception import MuException
 
+"""
+Suported system names
+"""
+MU32_SYSTEM_NAME				= 'Mu32'
+MU32USB2_SYSTEM_NAME			= 'Mu32usb2'
+MU256_SYSTEM_NAME				= 'Mu256'
+MU1024_SYSTEM_NAME				= 'Mu1024'
+
 """
 Some constants
 """
@@ -133,12 +141,13 @@ DEFAULT_H5_FILE_DURATION			= 15*60									# Time duration of a complete H5 file
 DEFAULT_H5_COMPRESSING				= False									# Whether compression mode is On or Off
 DEFAULT_H5_COMPRESSION_ALGO 		= 'gzip'								# Compression algorithm (gzip, lzf, szip)
 DEFAULT_H5_GZIP_LEVEL 				= 4										# compression level for gzip algo (0 to 9, default 4) 
-
+DEFAULT_H5_DIRECTORY				= './'									# The default directory where H5 files are saved
 
 class MegaMicro:
 	"""
 	MegaMicro core abstract class
 	"""
+	_system = ''
 	_usb_vendor_id = 0
 	_usb_vendor_product = 0
 	_usb_bus_address = 0
@@ -174,6 +183,7 @@ class MegaMicro:
 	_transfer_thread = None
 	_transfer_thread_exception: MuException = None
 	_h5_recording = DEFAULT_H5_RECORDING
+	_h5_rootdir = DEFAULT_H5_DIRECTORY
 	_h5_file_duration  = DEFAULT_H5_FILE_DURATION
 	_h5_compressing = DEFAULT_H5_COMPRESSING
 	_h5_compression_algo = DEFAULT_H5_COMPRESSION_ALGO
@@ -198,10 +208,11 @@ class MegaMicro:
 		self._usb_bus_address = usb_bus_address
 		self._pluggable_beams_number = pluggable_beams_number
 
+		log.info( f" .MegaMicro({self._system}): created" )
+
 
 	def __del__( self ):
-		log.info( '-'*20 )
-		log.info('MegaMicro: end')
+		log.info( f" .MegaMicro({self._system}): destroyed" )
 
 	@property
 	def signal_q( self ):
@@ -304,6 +315,7 @@ class MegaMicro:
 			'transfers_count': self._transfers_count,
 			'block': self._block,
 			'h5_recording': self._h5_recording,
+			'h5_rootdir': self._h5_rootdir,
 			'h5_dataset_duration': self._h5_dataset_duration,
 			'h5_file_duration': self._h5_file_duration,
 			'h5_compressing': self._h5_compressing,
@@ -333,6 +345,7 @@ class MegaMicro:
 			'transfers_count': self._transfers_count,
 			'block': self._block,
 			'h5_recording': self._h5_recording,
+			'h5_rootdir': self._h5_rootdir,
 			'h5_dataset_duration': self._h5_dataset_duration,
 			'h5_file_duration': self._h5_file_duration,
 			'h5_compressing': self._h5_compressing,
@@ -344,6 +357,10 @@ class MegaMicro:
 	def h5_recording( self ):
 		return self._h5_recording
 
+	@property
+	def h5_rootdir( self ):
+		return self._h5_rootdir
+
 	@property
 	def h5_dataset_duration( self ):
 		return self._h5_dataset_duration
@@ -893,14 +910,155 @@ class MegaMicro:
 	
 
 	def run_setargs( self, kwargs ):
+		"""
+		Set MegaMicro property values with priority order given to arguments (if given), parameter list (if given) and then defaults values.
+
+		:param clockdiv: decide for the sampling frequency. The sampling frequency is given by int( 500000/( clockdiv+1 ) ).Ex: 0x09 set for 50kHz
+		:param buffers_number: the number of buffers used by the USB device for the data bulk transfer. It can be set from 1 to n>1 (default is given by MU_DEFAULT_BUFFERS_NUMBER)
+		:param buffer_length: the number of samples that will be sent for each microphone by the Mu32 system in each transfer buffer. buffers_number_number and buffer_length have effects on latence and real time capabilities
+		:param duration: the desired recording time in seconds
+		"""
 
+		"""
+		Set default values
+		"""
+		sampling_frequency = DEFAULT_SAMPLING_FREQUENCY
+		buffers_number = DEFAULT_BUFFERS_NUMBER
+		buffer_length = DEFAULT_BUFFER_LENGTH
+		duration = DEFAULT_DURATION
+		datatype = DEFAULT_DATATYPE
+		mems = DEFAULT_ACTIVATED_MEMS
+		analogs = DEFAULT_ACTIVATED_ANALOG
+		counter = DEFAULT_ACTIVATED_COUNTER
+		counter_skip = DEFAULT_COUNTER_SKIPPING
+		status = DEFAULT_ACTIVATED_STATUS
+		block = DEFAULT_BLOCK_FLAG
+		callback_fn = None
+		post_callback_fn = None
+		h5_recording = DEFAULT_H5_RECORDING
+		h5_rootdir = DEFAULT_H5_DIRECTORY
+		h5_dataset_duration = DEFAULT_H5_SEQUENCE_DURATION
+		h5_file_duration = DEFAULT_H5_FILE_DURATION
+		h5_compressing = DEFAULT_H5_COMPRESSING
+		h5_compression_algo = DEFAULT_H5_COMPRESSION_ALGO
+		h5_gzip_level = DEFAULT_H5_GZIP_LEVEL
+
+		if len( kwargs ) == 0:
+			"""
+			No argument provided -> perform autotest
+			"""
+			post_callback_fn = self.autotest
+			mems = [i for i in range( self._pluggable_beams_number*MU_BEAM_MEMS_NUMBER )]
+
+
+		if 'parameters' in kwargs:
+			"""
+			Update with parameter list dicionnary content
+			"""
+			parameters = kwargs['parameters']
+			if 'sampling_frequency' in parameters:
+				sampling_frequency = parameters.get( 'sampling_frequency' )
+			if 'buffers_number' in parameters:
+				buffers_number = parameters.get( 'buffers_number' )
+			if 'buffer_length' in parameters:
+				buffer_length = parameters.get( 'buffer_length' )
+			if 'duration' in parameters:
+				duration = parameters.get( 'duration' )
+			if 'datatype' in parameters:
+				datatype = parameters.get( 'datatype' )
+			if 'mems' in parameters:
+				mems = parameters.get( 'mems' )
+			if 'analogs' in parameters:
+				analogs = parameters.get( 'analogs' )
+			if 'counter' in parameters:
+				counter = parameters.get( 'counter' )		
+			if 'counter_skip' in parameters:
+				counter_skip = parameters.get( 'counter_skip' )		
+			if 'status' in parameters:
+				status = parameters.get( 'status' )			
+			if 'block' in parameters:
+				block = parameters.get( 'block' )
+			if 'h5_recording' in parameters:
+				h5_recording = parameters.get( 'h5_recording' )
+			if 'h5_rootdir' in parameters:
+				h5_rootdir = parameters.get( 'h5_rootdir' )	
+			if 'h5_seq_duration' in parameters:
+				h5_dataset_duration = parameters.get( 'h5_dataset_duration' )
+			if 'h5_file_duration' in parameters:
+				h5_file_duration = parameters.get( 'h5_file_duration' )	
+			if 'h5_compressing' in parameters:
+				h5_compressing = parameters.get( 'h5_compressing' )	
+			if 'h5_compression_algo' in parameters:
+				h5_compression_algo = parameters.get( 'h5_compression_algo' )	
+			if 'h5_gzip_level' in parameters:
+				h5_gzip_level = parameters.get( 'h5_gzip_level' )
+		
+		"""
+		Update with run() arguments
+		"""
+		if 'sampling_frequency' in kwargs: sampling_frequency = kwargs['sampling_frequency']
+		if 'buffers_number' in kwargs: buffers_number = kwargs['buffers_number']
+		if 'buffer_length' in kwargs: buffer_length = kwargs['buffer_length']
+		if 'duration' in kwargs: duration = kwargs['duration']
+		if 'datatype' in kwargs: datatype = kwargs['datatype']
+		if 'mems' in kwargs: mems = kwargs['mems']
+		if 'analogs' in kwargs: analogs = kwargs['analogs']
+		if 'counter' in kwargs: counter = kwargs['counter']
+		if 'counter_skip' in kwargs: counter_skip = kwargs['counter_skip']
+		if 'status' in kwargs: status = kwargs['status']
+		if 'post_callback_fn' in kwargs: post_callback_fn = kwargs['post_callback_fn']
+		if 'callback_fn' in kwargs: callback_fn = kwargs['callback_fn']
+		if 'block' in kwargs: block = kwargs['block']
+		if 'h5_recording' in kwargs: h5_recording = kwargs['h5_recording']
+		if 'h5_rootdir' in kwargs: h5_rootdir = kwargs['h5_rootdir']
+		if 'h5_dataset_duration' in kwargs: h5_dataset_duration = kwargs['h5_dataset_duration']
+		if 'h5_file_duration' in kwargs: h5_file_duration = kwargs['h5_file_duration']
+		if 'h5_compressing' in kwargs: h5_compressing = kwargs['h5_compressing']
+		if 'h5_compression_algo' in kwargs: h5_compression_algo = kwargs['h5_compression_algo']
+		if 'h5_gzip_level' in kwargs: h5_gzip_level = kwargs['h5_gzip_level']
+
+		"""
+		Set atributes
+		"""
+		self._clockdiv = max( int( 500000 / sampling_frequency ) - 1, 9 )
+		self._sampling_frequency = 500000 / ( self._clockdiv + 1 )
+		self._buffer_length = buffer_length
+		self._buffers_number = buffers_number
+		self._datatype = datatype
+		self._buffer_duration = self._buffer_length / self._sampling_frequency
+		self._duration = duration
+		self._mems = mems
+		self._mems_number = len( self._mems )
+		self._analogs = analogs
+		self._analogs_number = len( self._analogs )
+		self._counter = counter
+		self._counter_skip = counter_skip
+		self._status = status
+		self._channels_number = self._mems_number + self._analogs_number + self._counter + self._status
+		self._buffer_words_length = self._channels_number*self._buffer_length
+		self._transfers_count = int( ( self._duration * self._sampling_frequency ) // self._buffer_length )
+		self._post_callback_fn = post_callback_fn
+		self._callback_fn = callback_fn
+		self._block = block
+		self._h5_recording = h5_recording
+		self._h5_rootdir = h5_rootdir
+		self._h5_dataset_duration = h5_dataset_duration
+		self._h5_file_duration = h5_file_duration
+		self._h5_dataset_number = int( h5_file_duration // h5_dataset_duration )
+		self._h5_compressing = h5_compressing
+		self._h5_compression_algo = h5_compression_algo
+		self._h5_gzip_level = h5_gzip_level
+
+
+	def run_setargs_old( self, kwargs ):
 		"""
 		Set MegaMicro property values according default, input arguments end/or parameter list given as argument 
-		----------
+
 		* clockdiv decide for the sampling frequency. The sampling frequency is given by int( 500000/( clockdiv+1 ) ).Ex: 0x09 set for 50kHz
 		* buffers_number is the number of buffers used by the USB device for the data bulk transfer. It can be set from 1 to n>1 (default is given by MU_DEFAULT_BUFFERS_NUMBER).
 		* buffer_length is the number of samples that will be sent for each microphone by the Mu32 system in each transfer buffer. buffers_number_number and buffer_length have effects on latence and real time capabilities. 
 		* duration is the desired recording time in seconds
+
 		Setting buffers_number to 1 should be used only for autotest purpose since it cannot ensure real time processing.
 		The more buffers_number is the more real time can be ensured without timeout or data flow breaks problems.
 		Conversely latency is increased.
@@ -936,6 +1094,8 @@ class MegaMicro:
 		else: parameters=kwargs.get('parameters')
 		if kwargs.get('h5_recording') is None: h5_recording=DEFAULT_H5_RECORDING
 		else: h5_recording=kwargs.get('h5_recording')
+		if kwargs.get('h5_rootdir') is None: h5_rootdir=DEFAULT_H5_DIRECTORY
+		else: h5_rootdir=kwargs.get('h5_rootdir')
 		if kwargs.get('h5_dataset_duration') is None: h5_dataset_duration=DEFAULT_H5_SEQUENCE_DURATION
 		else: h5_dataset_duration=kwargs.get('h5_dataset_duration')
 		if kwargs.get('h5_file_duration') is None: h5_file_duration=DEFAULT_H5_FILE_DURATION
@@ -971,6 +1131,7 @@ class MegaMicro:
 			status = DEFAULT_ACTIVATED_STATUS
 			block = DEFAULT_BLOCK_FLAG
 			h5_recording = DEFAULT_H5_RECORDING
+			h5_rootdir = DEFAULT_H5_DIRECTORY
 			h5_dataset_duration = DEFAULT_H5_SEQUENCE_DURATION
 			h5_file_duration = DEFAULT_H5_FILE_DURATION
 			h5_compressing = DEFAULT_H5_COMPRESSING
@@ -1003,7 +1164,9 @@ class MegaMicro:
 			if 'block' in parameters:
 				block = parameters.get( 'block' )
 			if 'h5_recording' in parameters:
-				h5_recording = parameters.get( 'h5_recording' )	
+				h5_recording = parameters.get( 'h5_recording' )
+			if 'h5_rootdir' in parameters:
+				h5_rootdir = parameters.get( 'h5_rootdir' )	
 			if 'h5_seq_duration' in parameters:
 				h5_dataset_duration = parameters.get( 'h5_dataset_duration' )
 			if 'h5_file_duration' in parameters:
@@ -1036,6 +1199,7 @@ class MegaMicro:
 		self._callback_fn = callback_fn
 		self._block = block
 		self._h5_recording = h5_recording
+		self._h5_rootdir = h5_rootdir
 		self._h5_dataset_duration = h5_dataset_duration
 		self._h5_file_duration = h5_file_duration
 		self._h5_dataset_number = int( h5_file_duration // h5_dataset_duration )
@@ -1057,10 +1221,17 @@ class MegaMicro:
 		if self._counter_skip and not self._counter:
 			log.warning( 'Cannot skip counter in the absence of counter (counter flag is off)' )
 
+		if not os.path.exists( self._h5_rootdir ):
+			log.warning( f"No H5 path `{self._h5_rootdir}` found in file system. Adjust to default" )
+			self._h5_rootdir = DEFAULT_H5_DIRECTORY
+
 
 	def run( self, **kwargs ):
 		"""
 		Run is a generic acquisition method that get signals from the activated MEMs
+
+		:param sampling_fequency: sampling frequency. Default is set to max value 50000Hz
+		:type sampling_frequency: float
 		"""
 
 		self.run_setargs( kwargs )
@@ -1069,15 +1240,14 @@ class MegaMicro:
 			"""
 			Do some controls and print recording parameters
 			"""
-			if self._analogs_number > 0:
-				log.warning( f"Mu32: {self._analogs_number} analogs channels were activated while they are not supported on Mu32 device -> unselecting")
+			if self._system==MU32_SYSTEM_NAME and self._analogs_number > 0:
+				log.warning( f"Mu32: {self._analogs_number} analogs channels were activated while they are not supported on {self._system} device -> unselecting")
 				self._analogs = []
 				self._analogs_number = 0
 				self._channels_number = self._mems_number + self._analogs_number + self._counter + self._status
 				self._buffer_words_length = self._channels_number*self._buffer_length
 
-			log.info( 'MegaMicro: Start running acquisition...')
-			log.info( '-'*20 )
+			log.info( f" .MegaMicro({self._system}): start running acquisition..." )
 
 			if self._datatype != 'int32' and self._datatype != 'float32':
 				raise MuException( 'Unknown datatype [%s]' % self._datatype )
@@ -1155,6 +1325,7 @@ class MegaMicro:
 		log.info( f" .minimal transfers count: {self._transfers_count}" )
 		log.info( f" .multi-threading execution mode: {not self._block}" )
 
+
 		if self._h5_recording:
 			log.info( f" .H5 recording: ON" )
 			self.h5_log_info()
@@ -1171,7 +1342,7 @@ class MegaMicro:
 				skip_on_error=True,
 			)
 			if handle is None:
-				self._transfer_thread_exception = MuException( 'USB3 device is not present or user is not allowed to access device' )
+				self._transfer_thread_exception = MuException( f"USB3 device for {self._system} system is not present or user is not allowed to access device" )
 				return
 
 			try:
@@ -1411,7 +1582,7 @@ class MegaMicro:
 		"""
 		with h5py.File( filename, "w" ) as f:
 			# creata the data set:
-			dataset = f.create_dataset( "megamicro", ( self.mems_number, np.size(signal, 1) ), dtype=self.datatype )
+			dataset = f.create_dataset( "megamicro", ( self._channels_number - int(self._counter and self._counter_skip), np.size(signal, 1) ), dtype=self.datatype )
 
 			# write data:
 			dataset[:] = signal
@@ -1422,40 +1593,43 @@ class MegaMicro:
 
 	def h5_log_info( self ):
 		"""
-		Init H5 data recording
+		Log info about H5 file system
 		"""
 		h5_dataset_length = int( self._h5_dataset_duration * self._sampling_frequency )
 		h5_dataset_size = h5_dataset_length * self.channels_number * MU_TRANSFER_DATAWORDS_SIZE
 		h5_file_samples_number = h5_dataset_length * self._h5_dataset_number
-		log.info( f" .dataset length is: {h5_dataset_length} samples ({self._h5_dataset_duration}s)" )
-		log.info( f" .dataset size: {h5_dataset_size/1024/1024} Mo" )
-		log.info( f" .file maximum length: {h5_file_samples_number} samples ({self._h5_file_duration}s)")
+		log.info( f" .H5 directory: `{self._h5_rootdir}`" )
+		log.info( f" .H5 dataset length is: {h5_dataset_length} samples ({self._h5_dataset_duration}s)" )
+		log.info( f" .H5 dataset size: {h5_dataset_size/1024/1024} Mo" )
+		log.info( f" .H5 file maximum length: {h5_file_samples_number} samples ({self._h5_file_duration}s)")
 		if self._h5_compressing:
 			log.info( f" .H5 compression: ON (algo is {self._h5_compression_algo})" )
 			if self._h5_compression_algo == 'gzip':
-				log.info( f" .Compression level (0 to 9): {self._h5_gzip_level}")
+				log.info( f" .H5 compression level (0 to 9): {self._h5_gzip_level}")
 		else:
 			log.info( f" .H5 compression: OFF" )
 
 
 	def h5_init( self ):
 		"""
+		Init H5 file system.
 		Organization of H5 file is as follows (fs of 50000Hz with 32 MEMs are values taken as reference):
-		- transfer buffers are saved into dataset of 'h5_dataset_length' buffers
-		- dataset are organized into groups of 'h5_group_length' datasets
-		- groups are stored into h5 files of 'h5_file_length' groups
-		These h5 parameters are MegaMicro parameters that have default values (100 buffers, 60 dataset, 60 groups). 
+
+		* transfer buffers are saved into dataset of ``_h5_dataset_duration`` duration
+		* datasets are stored into h5 files of '_h5_file_duration' duration
+
+		These h5 parameters are MegaMicro parameters that have default values (1 second datasets, 15 minutes files,
+		see ``core_base.DEFAULT_H5_SEQUENCE_DURATION`` and ``core_base.DEFAULT_H5_FILE_DURATION`` parameters).
 		With buffers of 512 samples each, it means that a complete file stores about one minute data (22Go at 50000Hz).
 		Files can be fractionned into subfiles to ensure safe network transfer. 
 		Internal organization is performed so as to rebuild bigger file from those subfiles.
-		Dataset are stored in files with their specific path:
-		- simulation_code/station_code/group_num/dataset_num/[mems|fft|doa|...]/data
+		H5 files are stored in the H5 root directory which default value is the current working directory
 		"""
 
 		"""
 		Create buffer and init first H5 file 
 		"""
-		log.info( ' .Init H5 recording process...' )
+		log.info( ' .H5 init recording process...' )
 		try:
 			self._h5_dataset_number = int( self._h5_file_duration // self._h5_dataset_duration )
 			self._h5_dataset_length = int( self._h5_dataset_duration * self._sampling_frequency )
@@ -1472,7 +1646,8 @@ class MegaMicro:
 		date = datetime.now()
 		timestamp0 = date.timestamp()
 		date0str = datetime.strftime(date, '%Y-%m-%d %H:%M:%S.%f')
-		filename = 'mu5h-' + f"{date.year}{date.month:02}{date.day:02}-{date.hour:02}{date.minute:02}{date.second:02}" + '.h5'
+		abs_path = os.path.abspath( self._h5_rootdir )
+		filename = os.path.join( abs_path, 'mu5h-' + f"{date.year}{date.month:02}{date.day:02}-{date.hour:02}{date.minute:02}{date.second:02}" + '.h5' )
 		self._h5_current_file = h5py.File( filename, "w" )
 		
 		self._h5_current_group = self._h5_current_file.create_group( 'muh5' )
@@ -1487,22 +1662,24 @@ class MegaMicro:
 		self._h5_current_group.attrs['datatype'] = self._datatype
 		self._h5_current_group.attrs['mems'] = np.array( self._mems )
 		self._h5_current_group.attrs['mems_number'] = self._mems_number
+		self._h5_current_group.attrs['analogs'] = np.array( self._analogs )
+		self._h5_current_group.attrs['analogs_number'] = self._analogs_number
 		self._h5_current_group.attrs['counter'] = self._counter
 		self._h5_current_group.attrs['counter_skip'] = self._counter_skip
 		self._h5_current_group.attrs['comment'] = ''
 		if self._h5_compressing:
-			self._h5_parameters['compression'] = self._h5_compression_algo
+			self._h5_current_group.attrs['compression'] = self._h5_compression_algo
 		else:
 			self._h5_current_group.attrs['compression'] = False
 
 		self._h5_dataset_index = 0
-		log.info( f" .Create new H5 file [{filename}]" )
+		log.info( f" .H5 create new file [{filename}]" )
 
 	def h5_write_mems( self, signal, timestamp ):
 		"""
-		Write transfer buffer in local cache buffer and tranfer local buffer to H5 file 
+		Write transfer buffer in local cache and tranfer local to H5 file 
 		! Beware that this function is not thread safe !
-		! it should be re-writen or writen outside of the acquisition thread ! 
+		! it should be re-writen or writen outside the acquisition thread ! 
 		"""
 
 		if self._h5_buffer_index + self._buffer_length < self._h5_dataset_length:
