@@ -38,17 +38,17 @@ import threading
 import json
 import numpy as np
 import queue
-from time import sleep
+from time import sleep, time
 
-from mu32.log import logging, mulog as log, mu32log		# mu32log for backward compatibility
+from mu32.log import logging, mulog as log, DEBUG_MODE, mu32log		# mu32log for backward compatibility
 from mu32.exception import Mu32Exception, MuException
 from mu32.core_base import MegaMicro, MU_TRANSFER_DATAWORDS_SIZE, MU_BEAM_MEMS_NUMBER
 
 
 DEFAULT_H5_PLAY_FILENAME			= './'
+PROCESSING_DELAY_RATE				= 1/10							# computing delay rate relative to transfer buffer duration
 
-
-log.setLevel( logging.INFO )
+#log.setLevel( logging.INFO )
 
 class MuH5( MegaMicro ):
 	"""
@@ -70,7 +70,36 @@ class MuH5( MegaMicro ):
 	def h5_files( self ):
 		return self._h5_files
 
+	@property
+	def parameters( self ):
+		parameters = dict()
+		if len( self._h5_files ) > 0:
+			for index, filename in enumerate( self._h5_files ):
+				with h5py.File( filename, 'r' ) as file:
+					if not file['muh5']:
+						"""
+						It seems not to be a MegaMicro H5 compatible file
+						"""
+						continue
+					else:
+						group = file['muh5']
+						parameters.update( {filename:  
+							dict( zip( group.attrs.keys(), group.attrs.values() ) )
+						} )
+
+		return parameters
+
+
 	def __init__( self, filename=DEFAULT_H5_PLAY_FILENAME ):
+
+		self._system = 'MuH5'
+		super().__init__( 
+			usb_vendor_id=0x00, 
+			usb_vendor_product=0x00,
+			usb_bus_address = 0x00,
+			pluggable_beams_number = 0
+		)
+
 		"""
 		Get H5 file or directory.
 		"""
@@ -87,8 +116,8 @@ class MuH5( MegaMicro ):
 
 
 	def __del__( self ):
-		log.info( '-'*20 )
 		log.info('MuH5: end')
+		log.info( '-'*20 )
 
 	def check_usb( self, verbose=True ):
 		pass
@@ -106,21 +135,14 @@ class MuH5( MegaMicro ):
 		Set specific arguments for playing H5 files
 		"""
 		h5_play_filename = None
-		if kwargs.get('h5_play_filename') is not None:
-			h5_play_filename=kwargs.get('h5_play_filename')
-		if kwargs.get('h5_start_time') is None: h5_start_time = 0
-		else: h5_start_time = kwargs.get('h5_start_time')
-		if kwargs.get('parameters') is None: parameters = None
-		else: parameters = kwargs.get('parameters')
-
-		if parameters is not None:
-			h5_play_filename = None
-			h5_start_time = 0
+		h5_start_time = 0
 
+		if 'parameters' in kwargs:
+			parameters = kwargs['parameters']
 			if 'h5_play_filename' in parameters:
-				h5_play_filename = parameters['parameters']
-			if 'h5_start_time' in parameters:
-				h5_start_time = parameters['h5_start_time']			
+				h5_play_filename = parameters['h5_play_filename']
+			if 'h5_start_time' in parameters and parameters['h5_start_time'] is not None:
+				h5_start_time = parameters['h5_start_time']		
 
 		if h5_play_filename is not None:
 			"""
@@ -143,9 +165,8 @@ class MuH5( MegaMicro ):
 		self._h5_start_time = h5_start_time
 
 
-
 	def run( self, **kwargs ):
-		if not self._h5_files:
+		if len( self._h5_files ) == 0:
 			log.error( f"No H5 file: unable to run. Aborting run service..." )
 			raise MuException( f"No H5 file: unable to run. Aborting run service..." )
 
@@ -155,8 +176,7 @@ class MuH5( MegaMicro ):
 		Start control thread.
 		Ctrl_thread() run and control the transfer process thread and manage H5 files
 		"""
-		log.info( f" .Processing transfer")
-		log.info( '-'*20 )
+		log.info( f" .Processing transfer...")
 
 		if self._block:
 			"""
@@ -184,53 +204,71 @@ class MuH5( MegaMicro ):
 			try:
 				self.transfer_loop()
 			except MuException as e:
-				log.critical( str( e ) )
+				log.critical( str( e ), exc_info=DEBUG_MODE )
 				self._h5_ctrl_thread_exception = e
 			except Exception as e:
-				log.critical( f"Unexpected error:{e}" )
+				log.critical( f"Unexpected error:{e}", exc_info=DEBUG_MODE )
 				self._h5_ctrl_thread_exception = e
+			except :
+				log.critical( f"Unexpected unknown system error", exc_info=DEBUG_MODE )
 
 		log.info( f" .End of control thread" )
 
 
+
+	def _process_transfert( self, data: np.ndarray ):
+		"""
+		Manage extracted data according user parameters
+		"""
+		if self._callback_fn != None:
+			"""
+			Call user callback processing function if any
+			"""
+			try:
+				self._callback_fn( self, data )
+			except KeyboardInterrupt as e:
+				log.info( ' .keyboard interrupt...' )
+				self._h5_playing = False
+			except Exception as e:
+				log.critical( f"Unexpected error {e}. Aborting..." )
+				self._h5_playing = False
+		else:
+			"""
+			Save to queue 
+			"""
+			self._signal_q.put( data )
+
+
 	def transfer_loop( self ):
 		"""
 		! Note that continuity between files is not managed
-		! Timing should be improved
-		! sub list of MEMs is not handled (eg [0, 1, 2] instead of complete list)
 		"""
 		with h5py.File( self._h5_current_filename, 'r' ) as self._h5_current_file:	
 			"""
 			Control whether H5 file is a MuH5 file
 			"""
 			if not self._h5_current_file['muh5']:
-				log.warning( f"{self._h5_current_file} seems not to be a MuH5 file: unrecognized format" )
-				raise Mu32Exception( f"{self._h5_current_file} seems not to be a MuH5 file: unrecognized format" )
+				raise Mu32Exception( f"{self._h5_current_filename} seems not to be a MuH5 file: unrecognized format" )
 
 			"""
 			get parameters values on H5 file
 			"""
 			group = self._h5_current_file['muh5']
 			self._h5_parameters = dict( zip( group.attrs.keys(), group.attrs.values() ) )
-			self._available_mems = self._h5_parameters['mems']
-			self._pluggable_beams_number = int( len( self._available_mems ) / MU_BEAM_MEMS_NUMBER )	
+
 			self.run_setargs( self._h5_run_kwargs )
 
 			"""
 			Perform controls on requested parameter values: see if they are in accordance with H5 file parameters
 			"""
-			error = False
 			if self._sampling_frequency != self._h5_parameters['sampling_frequency']:
-				error = MuException( f"MuH5: no available under/over sampling algorithm: requested sampling frequency of {self._sampling_frequency}Hz do not match recording one at {self._h5_parameters['sampling_frequency']}Hz" )
-				log.error( str( error ) )
-				raise error	
+				raise MuException( f"MuH5: no available under/over sampling algorithm: requested sampling frequency of {self._sampling_frequency}Hz do not match recording one at {self._h5_parameters['sampling_frequency']}Hz" )
 
 			if self._counter and not self._counter_skip and ( not self._h5_parameters['counter'] or ( self._h5_parameters['counter'] and self._h5_parameters['counter_skip'] ) ):
-				error = MuException( f"MuH5: Counter is requested but not available on H5 data" )
-				log.error( str( error ) )
-				raise error	
+				raise MuException( f"MuH5: Counter is requested but not available on H5 data" )
 
 			self._available_mems = list( self._h5_parameters['mems'] )
+			self._pluggable_beams_number = int( len( self._available_mems ) / MU_BEAM_MEMS_NUMBER )	
 			if self._mems_number == 0:
 				"""
 				Check activated MEMs. 
@@ -243,16 +281,21 @@ class MuH5( MegaMicro ):
 			activated_mems = np.array( self._mems )
 			mask = np.logical_not( np.isin( activated_mems , self._h5_parameters['mems'] ) )
 			if len( activated_mems[mask] ) > 0:
-				error = MuException( f"Some activated microphones ({activated_mems[mask]}) are not available on H5 file {self._h5_current_filename}. Available MEMs are: {self._h5_parameters['mems']} ")
-				log.error( str( error ) )
-				raise error	
+				"""
+				Some activated MEMs are not available in the H5 file -> abort
+				We could react differently by adapting but reponse would not respect user request.  
+				"""
+				raise MuException( f"Some activated microphones ({activated_mems[mask]}) are not available on H5 file {self._h5_current_filename}. Available MEMs are: {self._h5_parameters['mems']} ")
+
 
 			"""
 			Start playing
 			"""
 			log.info( f" .Reading H5 file {self._h5_current_filename}")
 			log.info( f" .{self._h5_parameters['duration']}s ({(self._h5_parameters['duration']/60):.02}min) of data in {self._h5_current_filename} H5 file" )
+			log.info( f" .starting time: {self._h5_start_time}s" )
 			log.info( f" .Available mems: {self._available_mems}" )
+			log.info( f" .whether counter available: {self._h5_parameters['counter'] and not self._h5_parameters['counter_skip']}" )
 			log.info( f" .desired recording duration: {self._duration} s" )
 			log.info( f" .minimal recording duration: {( self._transfers_count*self._buffer_length ) / self._sampling_frequency} s" )
 			log.info( f" .{self._mems_number} activated microphones" )
@@ -280,7 +323,6 @@ class MuH5( MegaMicro ):
 			"""
 			self._transfer_index = 0
 			self._h5_playing = True
-			skipping_counter = self._h5_parameters['counter'] and not self._h5_parameters['counter_skip'] and self._counter_skip
 			start_time = self._h5_start_time * self._h5_parameters['duration'] / 100
 
 			if start_time > 0:
@@ -301,19 +343,48 @@ class MuH5( MegaMicro ):
 				self._h5_dataset_index_ptr = 0
 
 			dataset = self._h5_current_file['muh5/' + str( self._h5_dataset_index ) + '/sig']
-			if skipping_counter:
-				transfer_buffer = np.array( dataset[:] )[1:self._h5_parameters['channels_number'],:]
+
+			"""
+			Set the mask for mems selecting
+			* mask: the binary mask for slecting channels to get
+			* masking: True if somme channels are masked, False for complete copy
+			* channels_number: selected microphones + counter if available and selected
+			"""
+			mask = list( np.isin( self._available_mems, self._mems ) )
+			if self._h5_parameters['counter'] and not self._h5_parameters['counter_skip']:
+				"""
+				H5 has counter
+				"""
+				if self._counter_skip:
+					"""
+					User want to skip it
+					"""
+					mask = [False] + mask
+				else:
+					mask = [True] + mask
+			channels_number = sum(mask)
+			masking = channels_number != len(mask)
+
+			if masking:
+				transfer_buffer = np.array( dataset[:] )[mask,:]
 			else:
 				transfer_buffer = np.array( dataset[:] )
 
+			time_start = time()
+			initial_time = time_start
+			processing_delay = self._buffer_duration * PROCESSING_DELAY_RATE
 			while self._h5_playing == True:
 
 				if self._h5_dataset_index_ptr + self._buffer_length <= self._h5_parameters['dataset_length']:
 					"""
-					There is enough data in current dataset: put in the queue
+					There is enough data in current dataset: process to transfert
 					"""
-					self._signal_q.put( transfer_buffer[:,self._h5_dataset_index_ptr:self._h5_dataset_index_ptr+self._buffer_length] )
+					if ( time() - time_start ) < self._buffer_duration - processing_delay:
+						sleep( self._buffer_duration-time()+time_start-processing_delay )
+					time_start = time()
+					self._process_transfert( transfer_buffer[:,self._h5_dataset_index_ptr:self._h5_dataset_index_ptr+self._buffer_length] )
 					self._h5_dataset_index_ptr += self._buffer_length
+					
 				else:
 					"""
 					No enough data in current dataset: open next dataset
@@ -325,14 +396,22 @@ class MuH5( MegaMicro ):
 						current_dataset_last_samples_number = self._h5_parameters['dataset_length'] - self._h5_dataset_index_ptr
 						buffer = transfer_buffer[:,self._h5_dataset_index_ptr:self._h5_dataset_index_ptr+self._h5_parameters['dataset_length']]
 						dataset = self._h5_current_file['muh5/' + str( self._h5_dataset_index ) + '/sig']
-						if skipping_counter:
-							transfer_buffer = np.array( dataset[:] )[1:self._h5_parameters['channels_number'],:]
+						if masking:
+							transfer_buffer = np.array( dataset[:] )[mask,:]
 						else:
 							transfer_buffer = np.array( dataset[:] )
 
 						new_dataset_first_samples_number = self._buffer_length - current_dataset_last_samples_number
 						buffer = np.append( buffer, transfer_buffer[:,:new_dataset_first_samples_number], axis=1 )
-						self._signal_q.put( buffer )
+
+						"""
+						Transfer buffer
+						"""
+						if ( time() - time_start ) < self._buffer_duration - processing_delay:
+							sleep( self._buffer_duration-time()+time_start - processing_delay )
+						time_start = time()
+						self._process_transfert( buffer )
+
 						self._h5_dataset_index_ptr = new_dataset_first_samples_number
 						self._h5_dataset_index += 1
 						log.info( f" .new dataset: [{self._h5_dataset_index}]" )
@@ -341,29 +420,38 @@ class MuH5( MegaMicro ):
 						No more dataset: save current buffer and stop playing
 						"""
 						buffer = transfer_buffer[:,self._h5_dataset_index_ptr:self._h5_parameters['dataset_length']]
-						buffer = np.append( buffer, np.zeros( (self._mems_number, self._buffer_length - self._h5_parameters['dataset_length'] + self._h5_dataset_index_ptr), dtype=np.int32), axis=1 )
-						self._signal_q.put( buffer )
+						buffer = np.append( buffer, np.zeros( (channels_number, self._buffer_length - self._h5_parameters['dataset_length'] + self._h5_dataset_index_ptr), dtype=np.int32), axis=1 )
+						
+						"""
+						Transfer buffer
+						"""
+						if time() - time_start < self._buffer_duration - processing_delay:
+							sleep( self._buffer_duration-time()+time_start - processing_delay )
+						time_start = time()
+						self._process_transfert( buffer )
+
 						self._h5_playing = False
 						log.info( f" .no more dataset: stop playing" )
 
-
 				self._transfer_index += 1
 				if self._transfers_count != 0 and  self._transfer_index >= self._transfers_count:
 					self._h5_playing = False
 
-				"""
-				Wait for real time: emit signal through the net to time interval equal to the buffer transfer duration
-				Should be improved by using time event emetter
-				"""
-				sleep( self._buffer_duration )
+			"""
+			Compute elasped time
+			"""
+			elapsed_time = time()-initial_time
 
 			"""
 			Call the final callback user function if any 
 			"""
 			if self._post_callback_fn != None:
 				self._post_callback_fn( self )
-
-
+			
+			if self._duration == 0:
+				log.info( f" .Elapsed time: {elapsed_time}s (H5 file duration was: {self._h5_parameters['duration']}s)")
+			else:
+				log.info( f" .Elapsed time: {elapsed_time}s (expected duration was: {self._duration}s)")
 
 
 	def wait( self ):
@@ -397,6 +485,41 @@ class MuH5( MegaMicro ):
 		self._h5_playing = False
 
 
+	def autotest( self, mu ):
+		""" 
+		Post processing callback function for autotest the MuH5 file
+		Call MegaMicro autotest function and add some H5 complements
+		"""
+		super().autotest( mu )
+
+		print( f"From H5 file: {self._h5_current_filename }")
+		"""
+		Control whether H5 file is a MuH5 file (it should...)
+		"""
+		if not self._h5_current_file['muh5']:
+			raise Mu32Exception( f"{self._h5_current_filename} seems not to be a MuH5 file: unrecognized format" )
+
+		"""
+		get parameters values on H5 file
+		"""
+		group = self._h5_current_file['muh5']
+		h5_parameters = dict( zip( group.attrs.keys(), group.attrs.values() ) )
+
+		print( '-'*20 )
+		print( f" .Date: {h5_parameters['date']}" )
+		print( f" .Duration: {h5_parameters['duration']}s ({(h5_parameters['duration']/60):.02}min) of data" )
+		print( f" .Dataset number: {h5_parameters['dataset_number']}" )
+		print( f" .Dataset length: {h5_parameters['dataset_length']} samples" )
+		print( f" .Dataset duration: {h5_parameters['dataset_duration']}s" )		
+		print( f" .Channels number: {h5_parameters['channels_number']} channels" )
+		print( f" .Sampling frequency: {h5_parameters['sampling_frequency']}" )
+		print( f" .Data type: {h5_parameters['datatype']}" )
+		print( f" .Recorded mems: {h5_parameters['mems']} ({h5_parameters['mems_number']} mems)" )
+		print( f" .Recorded analog channels: {h5_parameters['analogs']} ({h5_parameters['analogs_number']} channels)" )
+		print( f" .Counter: {h5_parameters['counter']}" )
+		print( f" .Counter skip: {h5_parameters['counter_skip']}" )
+		print( f" .Data compression: {h5_parameters['compression']}" )
+		print( '-'*20 )
 
 
 def main():
@@ -410,6 +533,3 @@ def main():
 
 if __name__ == "__main__":
 	main()
-
-
-#a = [ -3502.,  -3232.,  -6190.,  -6946.,  -6288.,  -7560.,  -7486.,  -6848.,  -3712., -3680.,  -2686.,  -2386.,   2296.,   5658.,   7584.,   9784.,   9208.,   9952., 6452.,   7018.,   5696.,   7590.,   4884.,   8022.,  11706.,  12034.,  14892., 15728.,  16466.,  17300.,  18816.,  16344.,  14904.,  13798.,  10414.,   6474., 6290.,   5226.,   8388.,   9272.,   8160.,  10264.,  12090.,  14362.,  14836., 21114.,  19828.,  19636.,  20940.,  20466.,  18298.,  17896.,  16010.,  12304., 14026.,  15140.,  17568.,  18378.,  17862.,  19724.,  17998.,  15140.,  11330., 8896.,   6768.,   4826.,    538.,  -3434.,  -2524.,  -2586.,   -754.,   -204., 528.,   -458.,  -3170.,  -4910.,  -5704., -11032., -14548., -15818., -15354., -16076., -17686., -20104., -20544., -17654., -18202., -17742., -15812., -13654., -14220., -14936., -16452., -18112., -18248., -17542., -19752., -18144., -19196., -17632.]
\ No newline at end of file
