@@ -85,7 +85,8 @@ def main():
 			mems=MEMS,					# activated mems	
 			duration = DURATION,
 			buffer_length = BUFFER_LENGTH,
-			buffers_number = BUFFERS_NUMBER
+			buffers_number = BUFFERS_NUMBER,
+			block=True,
 		)
 	except Mu32Exception as e:
 		print( 'aborting' )
@@ -103,7 +104,6 @@ def callback_bfm( mu32: Mu32, data: np.ndarray ):
 
 	powers, beams_number = beamformer.das_doa( 
 		G,
-		#data.reshape( mu32.buffer_length, mu32.mems_number ).T/0x7fff,
 		data * mu32.sensibility,
 		sf=SAMPLING_FREQUENCY, 
 		bfwin_duration=BFWIN_DURATION 
@@ -114,7 +114,7 @@ def callback_bfm( mu32: Mu32, data: np.ndarray ):
 	"""
 	for power, bar in zip( powers[:,0], bars ):
 		bar.set_height( power * GAIN )
-	plt.pause( 0.0000001 )
+	plt.pause( 1e-10 )
 
 
 def init_graph():
