@@ -27,6 +27,8 @@ Mu32 documentation is available on https://distalsense.io
 
 import logging
 
+DEBUG_MODE = True
+
 class Mu32Formatter(logging.Formatter):
 	"""Logging Formatter to add colors and count warning / errors"""
 
@@ -60,7 +62,7 @@ mulog_ch.setLevel( logging.DEBUG )
 mulog_ch.setFormatter( Mu32Formatter() )
 
 mulog = logging.getLogger( __name__ )
-mulog.setLevel( logging.WARNING )
 mulog.addHandler( mulog_ch )
+mulog.setLevel( logging.NOTSET )
 
 mu32log = mulog
